% Authors:
% Agostino Gibaldi
% PSPC-lab - Biophysical and Electronic Engineering Department - University of Genoa
% March 2010 
%
% VERGENCE VELOCITY CONTROL [deg/s] COMPUTED FROM A IMAGE STEREO PAIR. 
% OUTPUT:
%   delta_ni - VELOCITY OF THE VERGENCE ANGLE
% INPUT:
%   II - IMAGE STEREO PAIR IN GRAYSCALE [NxMx2], THE THIRD DIMENSION
%       INDICATE THE LEFT AND RIGHT IMAGES
%   ph_shifts - NAME OF THE PHASE SHIFT FILE FOR THE POPULATION (ph_shift7_f16.mat)
%   filter - NAME OF THE FILE WITH THE GABOR FILTER (Gt43B0.0208f0.063.mat)
%   verg_control - NAME OF THE FILE CONTAINIG THE WEIGHTS FOR THE VERGENCE
%       CONTROL (control1.mat)
%   fovea - STANDARD DEVIATION OF THE GAUSSIAN CENTERED IN FOVEA [deg]
%   focal_length - CAMERA FOCAL LENGTH [mm]
%   px_sze - HORIZONTAL AND VERTICAL SIZE OF THE PIXEL IN A 2x1 VECTOR [mm]
%   frame_rate - FRAME RATE OF THE CAMERA [Hz]

function delta_ni = verg_velocity_control(II, ph_shift, filter, verg_control,fovea,focal_length,px_sze,frame_rate)

%%%     ENERGY MODEL    %%%

II(isnan(II)) = 0;

[F] = filt_steer_or_gabor(II,filter);
CL=F{1}(:,:,:,1); %CL
SL=F{2}(:,:,:,1); %SL

G=shift_in_phase(F{1}(:,:,:,2),F{2}(:,:,:,2),ph_shift);
CR=G{1};
SR=G{2};

clear F G

[sy sx n_orient phase_num]=size(SR);

E=zeros(sy,sx,n_orient,phase_num);


for o = 1:n_orient
    for ph_n=1:phase_num
        E(:,:,o,ph_n) = sqrt((CL(:,:,o) + squeeze(CR(:,:,o,ph_n))).^2 + (SL(:,:,o) + squeeze(SR(:,:,o,ph_n))).^2);
    end
end



%%%      VERGENCE      %%%

load(verg_control)

for c=1:size(control,3)

    VC(:,:,c)=sum(sum(E.*permute((repmat(control(:,:,c),[1 1 sx sy])),[4 3 1 2]),4),3);

end

VC(isnan(VC))=0;

verg_area = sx*fovea/(atan(px_sze(1)*0.5*sx/focal_length)*180/pi);

VC=squeeze(sum(sum(VC.*repmat(fspecial('gaussian',[sy sx],verg_area),[1 1 size(VC,3)]))));

VC=VC./VC(7);

if (VC(5)>1.0)     % SHORT
    m=3*(VC(3)-VC(4));
else                % LONG
    m=6*(VC(1)-VC(2));
end

delta_ni=180/pi*atan(px_sze(1)*m/focal_length)./frame_rate;

